@@ -8,7 +8,6 @@ from ..version import __version__
 from .api import api_cli
 from .instance import instance_cli
 from .pipeline import pipeline_cli
-from .repository import repository_cli
 from .run import run_cli
 from .schedule import schedule_cli
 
@@ -17,7 +16,6 @@ def create_dagster_cli():
     commands = {
         'api': api_cli,
         'pipeline': pipeline_cli,
-        'repository': repository_cli,
         'run': run_cli,
         'instance': instance_cli,
         'schedule': schedule_cli,
