@@ -26,6 +26,14 @@ def make_new_backfill_id():
     return ''.join(random.choice(string.ascii_lowercase) for x in range(BACKFILL_TAG_LENGTH))
 
 
+def str_format_list(items):
+    return '[{items}]'.format(items=', '.join(['\'{item}\''.format(item=item) for item in items]))
+
+
+def str_format_set(items):
+    return '[{items}]'.format(items=', '.join(['\'{item}\''.format(item=item) for item in items]))
+
+
 def check_dagster_package_version(library_name, library_version):
     if __version__ != library_version:
         message = 'Found version mismatch between `dagster` ({}) and `{}` ({})'.format(
